Image 		       	*Image_Loneos_ToInternal (Image_Loneos *in, int Nvalues);
Image_Loneos 	       	*ImageInternalTo_Loneos (Image *in, int Nvalues);
Average                	*Average_Loneos_ToInternal (Average_Loneos *in, int Nvalues, SecFilt **primary);
Average_Loneos 	        *AverageInternalTo_Loneos (Average *in, int Nvalues, SecFilt *primary);
Measure                	*Measure_Loneos_ToInternal (Measure_Loneos *in, int Nvalues);
Measure_Loneos 	        *MeasureInternalTo_Loneos (Measure *in, int Nvalues);
SecFilt 	       	*SecFilt_Loneos_ToInternal (SecFilt_Loneos *in, int Nvalues);
SecFilt_Loneos 	       	*SecFiltInternalTo_Loneos (SecFilt *in, int Nvalues);
